local strings = {}
local focused = 1

local nfields = 25

local maxlen = 80
local fieldsep = 20 -- px
local samplerate = 4000
local minBufferTime = 0.040 -- seconds; a bit over 2 frames
local nsamples = samplerate * minBufferTime + 2^52 - 2^52
assert(nsamples % 2 == 0) -- must be even
local font = love.graphics.getFont()
local fieldMaxWidth = font:getWidth(("0"):rep(maxlen))
local fieldHeight = font:getHeight()

local playing = false

local source = love.audio.newQueueableSource(samplerate, 8, 1, 2)
local snd1 = love.sound.newSoundData(160, samplerate, 8, 1)
local snd2 = love.sound.newSoundData(160, samplerate, 8, 1)
--local snd3 = love.sound.newSoundData(160, samplerate, 8, 1)
local ptr = 1
local incomplete = false

function love.load()
  for i = 1, nfields do strings[i] = '' end
  strings[1] = '0'
  strings[2] = '1'
  strings[3] = '01'
  strings[4] = '001'
  strings[5] = '0001'
  strings[6] = '00001'
  strings[7] = '01001'
  strings[8] = '010001'
  strings[9] = '0010001'
  strings[10] = '0010101'
  strings[11] = '00100101'
  strings[12] = '010010001'
  strings[13] = '0010010001'
  strings[14] = '0010010101'
  strings[15] = '00100100101'
  strings[16] = '00101010101'
  strings[17] = '0010010010001'
  strings[18] = '0101001001001'
  strings[19] = '0101010101001'
  strings[20] = '01010101001001'

  strings[22] = '010101010101001'

  strings[24] = '01001010010100101'
end

local function sendNextSample()
  snd1, snd2 = snd2, snd1
  local start = 0
  if incomplete then
    snd1:setSample(0, 0)
    snd1:setSample(1, 0)
    start = 2
    ptr = ptr % #playing + 1
  end
  incomplete = false
  local i = start
  while i < nsamples - 1 do
    local bit = playing:byte(ptr) - 48
    snd1:setSample(i, 1)
    if bit == 1 then
      snd1:setSample(i + 1, 1)
      if i + 2 >= nsamples then
        incomplete = true
        break
      end
      snd1:setSample(i + 2, 0)
      snd1:setSample(i + 3, 0)
      i = i + 4
    else
      snd1:setSample(i + 1, 0)
      i = i + 2
    end
    ptr = ptr % #playing + 1
  end
  source:queue(snd1)
end

local function startPlaying()
  source:stop()
  playing = strings[focused]
  if playing == "" then
    playing = false
    return
  end
  ptr = 1
  sendNextSample()
  sendNextSample()
  source:play()
end

function love.keypressed(k, s)
  if k == "enter" or k == "return" then
    startPlaying()
    return
  end
  if k == "escape" then
    source:stop()
    playing = false
    return
  end
  if k == "q" and love.keyboard.isDown("lctrl", "rctrl") then
    love.event.quit()
    return
  end
  if k == "up" then
    focused = (focused - 2) % nfields + 1
    return
  end
  if k == "down" then
    focused = focused % nfields + 1
    return
  end
  if k == "backspace" then
    if strings[focused] ~= "" then
      strings[focused] = strings[focused]:sub(1, -2)
    end
  end
end

function love.textinput(t)
  if t ~= "1" and t ~= "0" then return end
  if #strings[focused] >= maxlen then return end
  strings[focused] = strings[focused] .. t
end

function love.update(dt)
  if playing then
    if source:getFreeBufferCount() > 0 then
      sendNextSample()
    end
  end
end


local function isPeriodic(s)
  local len = #s
  for i = 1, len - 1 do
    if len/i % 1 == 0 then
      if s:sub(1, i):rep(len/i) == s then
        return i
      end
    end
  end
  return false
end


function love.draw()
  love.graphics.print("Ctrl-Q to quit, Enter to play, Escape to stop, arrows to move, 0/1/backspace to edit", 15)
  for i = 1, nfields do
    if i == focused then
      love.graphics.setColor(.25, .25, .25)
      love.graphics.rectangle("fill", 10, i * fieldsep, fieldMaxWidth, fieldHeight)
      love.graphics.setColor(1, 1, 1)
    end
    love.graphics.setColor(.5, .5, .5)
    love.graphics.print(strings[i]:rep(maxlen):sub(1, maxlen), 10, i * fieldsep)
    love.graphics.setColor(1, 1, 1)
    local cycle = strings[i]
    local rep = isPeriodic(cycle)
    if rep then
      love.graphics.setColor(1, 0.3, 0.3)
      cycle = cycle:sub(1, rep)
    end
    love.graphics.print(#cycle + #(cycle:gsub('0','')), 15+fieldMaxWidth, i*fieldsep)
    love.graphics.print(strings[i], 10, i * fieldsep)
    if rep then
      love.graphics.setColor(1, 1, 1)
      love.graphics.print(cycle, 10, i * fieldsep)
    end
    love.graphics.setColor(1, 1, 1)

  end
end
